<?php

namespace Drupal\migrate_delete\Commands;

use Drupal\Core\Database\Connection;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;

/**
 * Drush commands for the Migrate Delete module.
 */
final class MigrateDeleteCommands extends DrushCommands {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a MigrateDeleteCommands object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(Connection $database, LoggerChannelFactoryInterface $logger_factory) {
    parent::__construct();
    $this->database = $database;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * List all migration-related database tables.
   */
  #[CLI\Command(name: 'migrate-delete:list', aliases: ['mdl'])]
  #[CLI\Help(description: 'List all migrate_map_* and migrate_message_* tables in the database.')]
  #[CLI\Usage(name: 'migrate-delete:list', description: 'Show all migration tables')]
  public function listTables() {
    $migration_tables = $this->getMigrationTables();

    if (empty($migration_tables)) {
      $this->output()->writeln('<info>No migration tables (migrate_map_* or migrate_message_*) found in the database.</info>');
      return;
    }

    // Group tables by type for better display.
    $map_tables = [];
    $message_tables = [];

    foreach ($migration_tables as $table) {
      if (strpos($table, 'migrate_map_') === 0) {
        $map_tables[] = $table;
      }
      elseif (strpos($table, 'migrate_message_') === 0) {
        $message_tables[] = $table;
      }
    }

    $this->output()->writeln(sprintf('<info>Found %d migration tables:</info>', count($migration_tables)));
    $this->output()->writeln('');

    if (!empty($map_tables)) {
      $this->output()->writeln(sprintf('<comment>Migration Map Tables (%d):</comment>', count($map_tables)));
      foreach ($map_tables as $table) {
        $this->output()->writeln('  - ' . $table);
      }
      $this->output()->writeln('');
    }

    if (!empty($message_tables)) {
      $this->output()->writeln(sprintf('<comment>Migration Message Tables (%d):</comment>', count($message_tables)));
      foreach ($message_tables as $table) {
        $this->output()->writeln('  - ' . $table);
      }
      $this->output()->writeln('');
    }

    $this->output()->writeln('<info>Use "drush migrate-delete:delete" to delete these tables.</info>');
  }

  /**
   * Delete all migration-related database tables.
   */
  #[CLI\Command(name: 'migrate-delete:delete', aliases: ['mdd'])]
  #[CLI\Help(description: 'Delete all migrate_map_* and migrate_message_* tables from the database.')]
  #[CLI\Usage(name: 'migrate-delete:delete', description: 'Delete all migration tables with confirmation')]
  #[CLI\Usage(name: 'migrate-delete:delete --yes', description: 'Delete all migration tables without confirmation')]
  public function deleteTables() {
    $migration_tables = $this->getMigrationTables();

    if (empty($migration_tables)) {
      $this->output()->writeln('<info>No migration tables found to delete.</info>');
      return;
    }

    // Show what will be deleted.
    $this->output()->writeln(sprintf('<comment>Found %d migration tables that will be deleted:</comment>', count($migration_tables)));

    // Group and display tables.
    $map_tables = array_filter($migration_tables, function ($table) {
      return strpos($table, 'migrate_map_') === 0;
    });
    $message_tables = array_filter($migration_tables, function ($table) {
      return strpos($table, 'migrate_message_') === 0;
    });

    if (!empty($map_tables)) {
      $this->output()->writeln(sprintf('  Map tables (%d): %s', count($map_tables), implode(', ', $map_tables)));
    }
    if (!empty($message_tables)) {
      $this->output()->writeln(sprintf('  Message tables (%d): %s', count($message_tables), implode(', ', $message_tables)));
    }

    $this->output()->writeln('');
    $this->output()->writeln('<error>WARNING: This action cannot be undone!</error>');

    // Check for confirmation.
    if (!$this->input()->getOption('yes')) {
      if (!$this->io()->confirm('Are you sure you want to delete all these migration tables?', FALSE)) {
        $this->output()->writeln('<info>Operation cancelled.</info>');
        return;
      }
    }

    // Perform deletion.
    $deleted_count = 0;
    $errors = [];

    $this->output()->writeln('<info>Deleting migration tables...</info>');

    foreach ($migration_tables as $table) {
      try {
        // Verify table name matches expected patterns for security.
        if (!$this->isValidMigrationTable($table)) {
          $errors[] = sprintf('Skipped invalid table name: %s', $table);
          continue;
        }

        $this->database->schema()->dropTable($table);
        $deleted_count++;
        $this->output()->writeln(sprintf('  ✓ Deleted: %s', $table));
      }
      catch (\Exception $e) {
        $error_msg = sprintf('Error deleting table %s: %s', $table, $e->getMessage());
        $errors[] = $error_msg;
        $this->output()->writeln(sprintf('  ✗ %s', $error_msg));
      }
    }

    // Summary output.
    $this->output()->writeln('');
    if ($deleted_count > 0) {
      $this->output()->writeln(sprintf('<info>Successfully deleted %d migration tables.</info>', $deleted_count));

      // Log the action.
      $logger = $this->loggerFactory->get('migrate_delete');
      $logger->info('Deleted @count migration tables via Drush command.', ['@count' => $deleted_count]);
    }

    if (!empty($errors)) {
      $this->output()->writeln('<error>Errors encountered:</error>');
      foreach ($errors as $error) {
        $this->output()->writeln('  ' . $error);
      }
    }

    if ($deleted_count === 0 && empty($errors)) {
      $this->output()->writeln('<comment>No tables were deleted.</comment>');
    }
  }

  /**
   * Get all migration-related tables from the database.
   *
   * @return array
   *   Array of table names that match migration patterns.
   */
  protected function getMigrationTables() {
    $tables = [];

    try {
      // Get database name.
      $database_name = $this->database->getConnectionOptions()['database'];

      // Query to get table names matching our patterns.
      $query = $this->database->select('information_schema.tables', 't')
        ->fields('t', ['table_name'])
        ->condition('t.table_schema', $database_name);

      // Add OR condition for both patterns.
      $or_condition = $query->orConditionGroup()
        ->condition('t.table_name', 'migrate_map_%', 'LIKE')
        ->condition('t.table_name', 'migrate_message_%', 'LIKE');
      $query->condition($or_condition);

      $result = $query->execute();

      while ($record = $result->fetchAssoc()) {
        $table_name = $record['table_name'];
        if ($this->isValidMigrationTable($table_name)) {
          $tables[] = $table_name;
        }
      }
    }
    catch (\Exception $e) {
      $this->output()->writeln(sprintf('<error>Error retrieving migration tables: %s</error>', $e->getMessage()));
    }

    sort($tables);
    return $tables;
  }

  /**
   * Validate that a table name matches expected migration table patterns.
   *
   * @param string $table_name
   *   The table name to validate.
   *
   * @return bool
   *   TRUE if the table name is a valid migration table.
   */
  protected function isValidMigrationTable($table_name) {
    // Only allow tables that start with migrate_map_ or migrate_message_.
    return (strpos($table_name, 'migrate_map_') === 0 ||
            strpos($table_name, 'migrate_message_') === 0) &&
           // Ensure no path traversal or SQL injection attempts.
           preg_match('/^migrate_(map|message)_[a-zA-Z0-9_]+$/', $table_name);
  }

}
