<?php

namespace Drupal\migrate_delete\Form;

use Drupal\Core\Url;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for deleting migration-related database tables.
 */
class MigrateDeleteForm extends FormBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a MigrateDeleteForm object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(Connection $database, MessengerInterface $messenger) {
    $this->database = $database;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceCircularReferenceException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('database'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'migrate_delete_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Get all migration-related tables.
    $migration_tables = $this->getMigrationTables();

    $form['description'] = [
      '#type' => 'markup',
      '#markup' => '<p>' . $this->t('This form allows you to delete all migration-related database tables. <strong>This action cannot be undone.</strong>') . '</p>',
    ];

    if (empty($migration_tables)) {
      $form['no_tables'] = [
        '#type' => 'markup',
        '#markup' => '<p>' . $this->t('No migration tables (migrate_map_* or migrate_message_*) were found in the database.') . '</p>',
      ];
      return $form;
    }

    $form['tables_info'] = [
      '#type' => 'details',
      '#title' => $this->t('Migration tables found (@count)', ['@count' => count($migration_tables)]),
      '#open' => TRUE,
    ];

    // Group tables by type for better display.
    $map_tables = [];
    $message_tables = [];

    foreach ($migration_tables as $table) {
      if (strpos($table, 'migrate_map_') === 0) {
        $map_tables[] = $table;
      }
      elseif (strpos($table, 'migrate_message_') === 0) {
        $message_tables[] = $table;
      }
    }

    if (!empty($map_tables)) {
      $form['tables_info']['map_tables'] = [
        '#type' => 'markup',
        '#markup' => '<h4>' . $this->t('Migration Map tables (@count)', ['@count' => count($map_tables)]) . '</h4><ul><li>' . implode('</li><li>', $map_tables) . '</li></ul>',
      ];
    }

    if (!empty($message_tables)) {
      $form['tables_info']['message_tables'] = [
        '#type' => 'markup',
        '#markup' => '<h4>' . $this->t('Migration Message tables (@count)', ['@count' => count($message_tables)]) . '</h4><ul><li>' . implode('</li><li>', $message_tables) . '</li></ul>',
      ];
    }

    $form['warning'] = [
      '#type' => 'markup',
      '#markup' => '<div class="messages messages--warning"><strong>' .
      $this->t('WARNING: Deleting these tables will permanently remove all migration tracking data and error messages. This action cannot be undone.') .
      '</strong></div>',
    ];

    $form['confirm_deletion'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('I understand that this will permanently delete all @count migration tables listed above and this action cannot be undone.', ['@count' => count($migration_tables)]),
      '#required' => TRUE,
      '#description' => $this->t('You must check this box to enable the delete button.'),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Delete all migration tables'),
      '#button_type' => 'danger',
      '#states' => [
        'disabled' => [
          ':input[name="confirm_deletion"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => Url::fromRoute('system.admin_config_system'),
      '#attributes' => ['class' => ['button']],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getValue('confirm_deletion')) {
      $form_state->setErrorByName('confirm_deletion', $this->t('You must confirm your intention to delete the migration tables.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $migration_tables = $this->getMigrationTables();

    if (empty($migration_tables)) {
      $this->messenger->addWarning($this->t('No migration tables found to delete.'));
      return;
    }

    $deleted_count = 0;
    $errors = [];

    foreach ($migration_tables as $table) {
      try {
        // Verify table name matches expected patterns for security.
        if (!$this->isValidMigrationTable($table)) {
          $errors[] = $this->t('Skipped invalid table name: @table', ['@table' => $table]);
          continue;
        }

        $this->database->schema()->dropTable($table);
        $deleted_count++;
      }
      catch (\Exception $e) {
        $errors[] = $this->t('Error deleting table @table: @error', [
          '@table' => $table,
          '@error' => $e->getMessage(),
        ]);
      }
    }

    // Provide feedback to the user.
    if ($deleted_count > 0) {
      $this->messenger->addStatus($this->t('Successfully deleted @count migration tables.', ['@count' => $deleted_count]));
    }

    if (!empty($errors)) {
      foreach ($errors as $error) {
        $this->messenger->addError($error);
      }
    }

    if ($deleted_count === 0 && empty($errors)) {
      $this->messenger->addWarning($this->t('No tables were deleted.'));
    }
  }

  /**
   * Get all migration-related tables from the database.
   *
   * @return array
   *   Array of table names that match migration patterns.
   */
  protected function getMigrationTables() {
    $tables = [];

    try {
      // Get database name.
      $database_name = $this->database->getConnectionOptions()['database'];

      // Query to get table names matching our patterns.
      $query = $this->database->select('information_schema.tables', 't')
        ->fields('t', ['table_name'])
        ->condition('t.table_schema', $database_name);

      // Add OR condition for both patterns.
      $or_condition = $query->orConditionGroup()
        ->condition('t.table_name', 'migrate_map_%', 'LIKE')
        ->condition('t.table_name', 'migrate_message_%', 'LIKE');
      $query->condition($or_condition);

      $result = $query->execute();

      while ($record = $result->fetchAssoc()) {
        $table_name = $record['table_name'];
        if ($this->isValidMigrationTable($table_name)) {
          $tables[] = $table_name;
        }
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('Error retrieving migration tables: @error', ['@error' => $e->getMessage()]));
    }

    sort($tables);
    return $tables;
  }

  /**
   * Validate that a table name matches expected migration table patterns.
   *
   * @param string $table_name
   *   The table name to validate.
   *
   * @return bool
   *   TRUE if the table name is a valid migration table.
   */
  protected function isValidMigrationTable($table_name) {
    // Only allow tables that start with migrate_map_ or migrate_message_.
    return (strpos($table_name, 'migrate_map_') === 0 ||
            strpos($table_name, 'migrate_message_') === 0) &&
           // Ensure no path traversal or SQL injection attempts.
           preg_match('/^migrate_(map|message)_[a-zA-Z0-9_]+$/', $table_name);
  }

}
